var voice_personals_services_module = angular.module('voicePersonals.services');

//an HTTP wrapper that blocks user input, handles errors, ensures connectivity, and validates for JSON responses
//currently assumes that every http request should block user input until completed
voice_personals_services_module.factory('vpHTTP', function($http, $q, $rootScope, $state, $timeout, vpLoading, vpSystemMessage, vpConfig  ) {


    //init
    vpSystemMessage.get();

    var issueErrorMessage = function(errorMsgAction) {
        $rootScope.showSystemMessage("Application Error", "Sorry!<br /><br />An application error has occurred while trying to "+errorMsgAction+'.<br /><br />Please try again later.', 'error');
        vpLoading.hideLoading();
    }

    //device connectivity check functions
    var isConnected = function() {   //want one that returns a promise

        var connectionCheckPromise = $q.defer();

        if (vpConfig.isCordova()) {  //only verify connectivity on devices
            var defer = $q.defer();
            timeoutDeviceCheck(0, defer);
            defer.promise.then(function(deviceConnectivity){
                if (deviceConnectivity)
                    connectionCheckPromise.resolve(deviceConnectivity);
                else
                    connectionCheckPromise.reject(deviceConnectivity);
            });
        } else   //not on device, browser based debugging
            connectionCheckPromise.resolve(true);

        return connectionCheckPromise.promise;

    }

    var timeoutDeviceCheck = function(delay, deferredEvent) {

        var timeoutPromise = $timeout(getDeviceConnectivity, delay);
        timeoutPromise.then(function(deviceCheckResult) {
            if (deviceCheckResult !== 'ionic not ready') {
                deferredEvent.resolve(deviceCheckResult);
            }
            else {
                timeoutDeviceCheck(500, deferredEvent);
            }
        });

    }

    var getDeviceConnectivity = function() {
        if (typeof navigator.connection !== 'undefined') { //ionic platform has not loaded yet
            return (navigator.connection.type != Connection.NONE) ? true : false;
        }
        else {  //browser base debugging
            return 'ionic not ready';
        }
    }

    //Setting this for every request will make the browser send an OPTIONS request to resolve CORS issues.
    //Found issue (before device ready maybe?) where vp config sw ver is not yet available.
    var getSharedHTTPHeaders = function() {
        return {ForceOptionPreflight: 'true', vpSoftwareVersion: vpConfig.getSoftwareVersion()};
    }

    var forceSoftwareUpdate = function() {
        var updateURIJS = vpConfig.getSoftwareUpdateURI();
        updateURIJS = "window.open('"+updateURIJS+"', '_system');";  //open in new system browser
        $rootScope.showSystemMessage("Software Update Required", '' +
            'Hurrah!<br /><br />' +
            'A new and improved version is available, to continue please update the application. <br /><br />' +
            '<a href="#" onclick="'+updateURIJS+'">Update Now</a><br /><br />');
    }


    //service
    return {

        //use no UI block to skip showLoading for background HTTP calls that user interface does not depend - I.E. polling for new messages
        //skipLoginRedirect set to false if API call does not require login
        get: function(url, errorMsgAction, noUIBlock, skipLoginRedirect) {

            var defer = $q.defer();

            if (! noUIBlock) vpLoading.showLoading();

            isConnected().then(function() {

                var httpPromise = $http.get(url, {headers: getSharedHTTPHeaders()});

                httpPromise.then(function (success) {

                        if (typeof success.data == 'string') {  //JSON parses did not return object or Array for some reason, error?
                            try {
                                JSON.parse(success.data);
                                defer.resolve(success);
                            } catch (json_parse_error) {
                                issueErrorMessage(errorMsgAction);
                                defer.reject(json_parse_error);
                            }
                        }
                        else defer.resolve(success);

                        if ( ! noUIBlock) vpLoading.hideLoading();
                    },
                    function (error) {

                        defer.reject(error);

                        if (error.status == 409 && error.data == 'User must be logged in.') { // /if not logged in redirect, unless told not to
                            if (skipLoginRedirect !== true) {
                                $state.go('tab.login');
                            }
                        }
                        else if (error.status == 409 && error.data == 'vpSoftwareVersionUpdateRequired')
                            forceSoftwareUpdate();
                        else {
                            issueErrorMessage(errorMsgAction);
                            //console.log("vpHTTP reported error: ", error);
                        }
                        if ( ! noUIBlock) vpLoading.hideLoading();

                    });
            }, function() { //no internet connectivity
                $rootScope.showSystemMessage("Internet Unavailable", "Sorry but an internet connection is required to continue.<br /><br />Please try again once a connection is established.", 'error');
                if (! noUIBlock) vpLoading.hideLoading();
                defer.reject("No internet connection.");
            });

            return defer.promise;

        },

        post: function(url, data, errMsgAction) {
            var defer = $q.defer();

            vpLoading.showLoading();

            isConnected().then(function() {
                var postPromise = $http.post(url, data, {headers: getSharedHTTPHeaders()});
                postPromise.then(function(success) {
                    defer.resolve(success);
                    vpLoading.hideLoading();
                }, function(error) {
                    if (error.status == 409) {   //custom user-facing server side error messages.
                        if (error.data == 'User must be logged in.')
                            $rootScope.showSystemMessage('Login Required', 'Please login to '+errMsgAction+'.', 'blocked', 'tab.login', 'Go to Login');
                        else if (error.status == 409 && error.data == 'vpSoftwareVersionUpdateRequired')
                            forceSoftwareUpdate();
                        else
                            $rootScope.showSystemMessage('Error', error.data, 'error');
                    }
                    else
                        issueErrorMessage(errMsgAction);
                    defer.reject(error);
                    vpLoading.hideLoading();
                });
            }, function() { //no internet connectivity
                $rootScope.showSystemMessage("Internet Unavailable", "Sorry but an internet connection is required to continue.<br /><br />Pleast try again once a connection is established.", 'error');
                vpLoading.hideLoading();
                defer.reject("No internet connection.");
            });

            return defer.promise;
        }

    }

});

